--[[
		    ----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----


                                        ######       ####     ######              ######        ####
                                      ##      ##   ##    ##   #     ##            #     ##    ##    ##
                                     ##           ##      ##  #      ##           #      ##  ##      ##
                                     #            #        #  #       #           #       #  #        #
                                     #            #        #  #       #           #       #  #        #
                                     #            #        #  #       #           #       #  #        #
                                     ##           ##      ##  #      ##           #      ##  ##      ##
                                      ##      ##   ##    ##   #     ##            #     ##    ##    ##
                                        ######       ####     ######              ######        ####


                                       #######      ####    #########  #########  ########  #######
                                       #     ##    ##  ##       #          #      #         #     ##
                                       #       #  #      #      #          #      #         #       #
                                       #     ##   #      #      #          #      #         #     ##
                                       #######    ########      #          #      ########  #######
                                       #     ##   #      #      #          #      #         #   #
                                       #       #  #      #      #          #      #         #    #
                                       #     ##   #      #      #          #      #         #     #
                                       #######    #      #      #          #      ########  #      #


		    ----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----


						    ----o----(||)----oo----(||)----o----

							  v2.03 3rd September 2017
						    Copyright (C) Taraezor / Chris Birch

						    ----o----(||)----oo----(||)----o----

]]
--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Localisation Setup
--		==================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local addonName, L = ...

local gmatch = string.gmatch
local insert = table.insert
local lower = string.lower
local match = string.match
local modf = math.modf
local random = random
local stringGsub = string.gsub
local sub = string.sub
local tostring = tostring
local type = type

local GetAchievementCriteriaInfo = GetAchievementCriteriaInfo
local GetAchievementNumCriteria = GetAchievementNumCriteria
local GetCurrentMapAreaID = GetCurrentMapAreaID
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel
local GetCursorPosition = GetCursorPosition
local GetTime = GetTime
local PlaySound = PlaySound
local StopSound = StopSound

local mapAreaID = 0
local totalSubZones = 7
local Ring = {}

--------------------------------------------------------------------------------------------------------------------------------------------
-- Print Colours - Blue theme
--------------------------------------------------------------------------------------------------------------------------------------------

local pc_colour_Prefix		= "\124cFF00BFFF"
local pc_colour_Highlight	= "\124cFF1E90FF"
local pc_colour_PlainText	= "\124cFF87CEFA"

local function printPC( message )
	DEFAULT_CHAT_FRAME:AddMessage( pc_colour_Prefix.. addonName.. ": ".. pc_colour_PlainText.. message.. "\124r" )
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Zone Data IDs
--		=========
--
--------------------------------------------------------------------------------------------------------------------------------------------

local mapAreaByID = {
	[4] = "zDurotar",
	[9] = "zMulgore",
	[11] = "zNorthernBarrens",
	[16] = "zArathiHighlands",
	[17] = "zBadlands",
	[19] = "zBlastedLands",
	[20] = "zTirisfalGlades",
	[21] = "zSilverpineForest",
	[22] = "zWesternPlaguelands",
	[23] = "zEasternPlaguelands",
	[24] = "zHillsbradFoothills",
	[26] = "zTheHinterlands",
	[27] = "zDunMorogh",
	[30] = "zElwynnForest",
	[32] = "zDeadwindPass",
	[34] = "zDuskwood",
	[35] = "zLochModan",
	[36] = "zRedridgeMountains",
	[37] = "zNorthernStranglethorn",
	[38] = "zSwampOfSorrows",
	[39] = "zWestfall",
	[40] = "zWetlands",
	[41] = "zTeldrassil",
	[42] = "zDarkshore",
	[43] = "zAshenvale",
	[61] = "zThousandNeedles",
	[81] = "zStonetalonMountains",
	[101] = "zDesolace",
	[121] = "zFeralas",
	[141] = "zDustwallowMarsh",
	[161] = "zTanaris",
	[181] = "zAzshara",
	[182] = "zFelwood",
	[201] = "zUnGoroCrater",
	[241] = "zMoonglade",
	[281] = "zWinterspring",
	[301] = "zStormwindCity",
	[321] = "zOrgrimmar",
	[341] = "zIronforge",
	[362] = "zThunderBluff",
	[381] = "zDarnassus",
	[382] = "zUndercity",
	[462] = "zEversongWoods",
	[463] = "zGhostlands",
	[464] = "zAzuremystIsle",
	[465] = "zHellfirePeninsula",
	[467] = "zZangarmarsh",
	[471] = "zTheExodar",
	[473] = "zShadowmoonValley",
	[476] = "zBloodmystIsle",
	[477] = "zNagrand",
	[478] = "zTerokkarForest",
	[479] = "zNetherstorm",
	[486] = "zBoreanTundra",
	[488] = "zDragonblight",
	[490] = "zGrizzlyHills",
	[491] = "zHowlingFjord",
	[492] = "zIcecrown",
	[493] = "zSholazarBasin",
	[495] = "zTheStormPeaks",
	[496] = "zZulDrak",
	[499] = "zIsleOfQuelDanas",
	[501] = "zWintergrasp",
	[504] = "zDalaran",
	[510] = "zCrystalsongForest",
	[541] = "zHrothgarsLanding",
	[544] = "zTheLostIsles",
	[606] = "zMountHyjal",
	[607] = "zSouthernBarrens",
	[610] = "zKelpTharForest",
	[614] = "zAbyssalDepths",
	[615] = "zShimmeringExpanse",
	[640] = "zDeepholm",
	[673] = "zTheCapeOfStranglethorn",
	[684] = "zRuinsOfGilneas",
	[685] = "zRuinsOfGilneasCity",
	[700] = "zTwilightHighlands",
	[709] = "zTolBaradPeninsula",
	[720] = "zUldum",
	[751] = "zTheMaelstrom",
	[864] = "zNorthshire",
	[888] = "zShadowglen",
	[890] = "zCampNarache",
	[891] = "zEchoIsles",
	[893] = "zSunstriderIsle",
	[894] = "zAmmenVale",
	[895] = "zNewTinkertown",
}

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Zone Data Bubbles
--		=================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local zones = {
	zAbyssalDepths = { L["Abyssal Depths"],
			{ ["X"] = 52.20, ["Y"] = 61.10, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {7} },
			{ ["X"] = 55.10, ["Y"] = 73.50, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {7} },
	},
	zAmmenVale = { L["Ammen Vale"],			-- No parenthesis
			{ ["X"] = 40.00, ["Y"] = 63.50, ["Size"] = 60, ["L"] = {7,28} },
			{ ["X"] = 56.30, ["Y"] = 39.30, ["Size"] = 60, ["L"] = {7,28} },
	},
	zArathiHighlands = { L["Arathi Highlands"],
			{ ["X"] = 21.30, ["Y"] = 54.80, ["Size"] = 30, ["L"] = {8,28,22} },
			{ ["X"] = 38.20, ["Y"] = 84.70, ["Size"] = 30, ["L"] = {8,28} },
			{ ["X"] = 63.50, ["Y"] = 68.30, ["Size"] = 70, ["L"] = {8,28,22} },
			{ ["X"] = 25.50, ["Y"] = 83.00, ["Size"] = 40, ["O"] = {21,19,11} },
	},
	zAshenvale = { L["Ashenvale"],
			{ ["X"] = 36.30, ["Y"] = 50.70, ["Size"] = 90, ["L"] = {8,28,34} },
			{ ["X"] = 49.1, ["Y"] = 70.20, ["Size"] = 100, ["L"] = {8,28,34} },
			{ ["X"] = 12.00, ["Y"] = 21.80, ["Size"] = 70, ["O"] = {19,21,11} },
	},
	zAzshara = { L["Azshara"],
			{ ["X"] = 35.60, ["Y"] = 74.75, ["Size"] = 45, ["L"] = {28,34,8,7} },
			{ ["X"] = 44.75, ["Y"] = 54.50, ["Size"] = 40, ["O"] = {21,19,25,11} },
	},
	zAzuremystIsle = { L["Azuremyst Isle"],
			{ ["X"] = 38.80, ["Y"] = 7.10, ["Size"] = 20, ["O"] = {25} },
			{ ["X"] = 36.00, ["Y"] = 53.00, ["Size"] = 60, ["O"] = {25} },
			{ ["X"] = 21.50, ["Y"] = 77.10, ["Size"] = 40, ["O"] = {25} },
			{ ["X"] = 20.37, ["Y"] = 54.19, ["Size"] = 20, ["O"] = {21,19,25} },
			{ ["X"] = 74.80, ["Y"] = 58.20, ["Size"] = 30, ["L"] = {7,28} },
			{ ["X"] = 81.70, ["Y"] = 47.90, ["Size"] = 30, ["L"] = {7,28} },
	},
	zBadlands = { L["Badlands"],
			{ ["X"] = 91.75, ["Y"] = 20.25, ["Size"] = 30, ["L"] = {29,33}, ["O"] = {22,19}, ["Notes"] = {12} },
			{ ["X"] = 91.70, ["Y"] = 33.30, ["Size"] = 30, ["O"] = {26,13,27,24,11} },
			{ ["X"] = 94.50, ["Y"] = 37.60, ["Size"] = 30, ["O"] = {26,13,27,24,11} },
	},
	zBlastedLands = { L["Blasted Lands"],		-- No parenthesis
			{ ["X"] = 41.00, ["Y"] = 79.30, ["Size"] = 40, ["L"] = {33}, ["O"] = {30,22}, ["Notes"] = {12} },
			{ ["X"] = 42.90, ["Y"] = 71.60, ["Size"] = 40, ["L"] = {33,22}, ["O"] = {30,22}, ["Notes"] = {12} },
			{ ["X"] = 49.40, ["Y"] = 70.00, ["Size"] = 30, ["L"] = {33,22}, ["O"] = {30,22}, ["Notes"] = {12} },
			{ ["X"] = 51.00, ["Y"] = 85.40, ["Size"] = 40, ["O"] = {5,27,28,16,3}, ["Notes"] = {13} },
			{ ["X"] = 72.70, ["Y"] = 71.40, ["Size"] = 60, ["O"] = {5,27,28,16,3}, ["Notes"] = {13} },
	},
	zBloodmystIsle = { L["Bloodmyst Isle"],
			{ ["X"] = 80.80, ["Y"] = 49.90, ["Size"] = 70, ["O"] = {21,25,19} },
			{ ["X"] = 74.30, ["Y"] = 35.20, ["Size"] = 40, ["O"] = {21,25,19} },
			{ ["X"] = 20.90, ["Y"] = 46.20, ["Size"] = 25, ["O"] = {21,25,19} },
			{ ["X"] = 23.20, ["Y"] = 72.70, ["Size"] = 70, ["O"] = {21,25,19} },
			{ ["X"] = 58.20, ["Y"] = 72.70, ["Size"] = 30, ["O"] = {21,25,19} },
	},
	zBoreanTundra = { L["Borean Tundra"],
			{ ["X"] = 89.30, ["Y"] = 56.10, ["Size"] = 70, ["O"] = {8,17,20} },
			{ ["X"] = 81.40, ["Y"] = 60.60, ["Size"] = 70, ["O"] = {8,17,20} },
			{ ["X"] = 79.50, ["Y"] = 52.50, ["Size"] = 30, ["O"] = {23,14,4,3,20} },
			{ ["X"] = 65.30, ["Y"] = 49.80, ["Size"] = 30, ["O"] = {23,14,3,4,20} },
			{ ["X"] = 51.20, ["Y"] = 44.80, ["Size"] = 70, ["L"] = {6,31,11}, ["O"] = {20} },
			{ ["X"] = 40.30, ["Y"] = 78.50, ["Size"] = 10, ["L"] = {6,31,11}, ["O"] = {20} },
			{ ["X"] = 40.20, ["Y"] = 75.70, ["Size"] = 30, ["O"] = {23,14,4,20,7}, ["L"] = {6} },
	},
	zCampNarache = { L["Camp Narache"],		-- No parenthesis
			{ ["X"] = 55.20, ["Y"] = 65.40, ["Size"] = 280, ["L"] = {7,28} },
	},
	zCrystalsongForest = { L["Crystalsong Forest"],
			{ ["X"] = 48.40, ["Y"] = 56.70, ["Size"] = 50, ["L"] = {19,6}, ["O"] = {20} },
			{ ["X"] = 17.20, ["Y"] = 16.20, ["Size"] = 40, ["L"] = {6,19}, ["O"] = {20} },
	},
	zDalaran = { L["Dalaran"],
			{ ["X"] = 54.10, ["Y"] = 66.90, ["Size"] = 20, ["L"] = {15}, ["Notes"] = {11} },
			{ ["X"] = 61.60, ["Y"] = 69.30, ["Size"] = 70, ["L"] = {37,39} },
			{ ["X"] = 66.40, ["Y"] = 60.40, ["Size"] = 70, ["L"] = {37,39} },
			{ ["X"] = 61.80, ["Y"] = 14.70, ["Size"] = 90, ["L"] = {35} },
			{ ["X"] = 45.10, ["Y"] = 65.60, ["Size"] = 70, ["L"] = {35} },
	},
	zDarkshore = { L["Darkshore"],
			{ ["X"] = 70.20, ["Y"] = 19.3, ["Size"] = 40, ["L"] = {28,7,34,8} },
			{ ["X"] = 54.40, ["Y"] = 31.90, ["Size"] = 30, ["L"] = {34}, ["O"] = {21,25,19,11} },
			{ ["X"] = 51.30, ["Y"] = 18.75, ["Size"] = 60, ["O"] = {6,21,25,19,11} },
			{ ["X"] = 35.50, ["Y"] = 44.20, ["Size"] = 60, ["O"] = {6,21,25,19,11} },
	},
	zDarnassus = { L["Darnassus"],			-- No parenthesis
			{ ["X"] = 48.80, ["Y"] = 61.30, ["Size"] = 60, ["L"] = {28,8,7} },
	},
	zDeadwindPass = { L["Deadwind Pass"],
			{ ["X"] = 51.80, ["Y"] = 72.60, ["Size"] = 40, ["L"] = {2,5,9} },
			{ ["X"] = 43.60, ["Y"] = 57.00, ["Size"] = 20, ["L"] = {2,5,9} },
	},
	zDeepholm = { L["Deepholm"],
			{ ["X"] = 68.00, ["Y"] = 69.20, ["Size"] = 35, ["L"] = {36,1,26} },
			{ ["X"] = 72.75, ["Y"] = 63.00, ["Size"] = 50, ["L"] = {36,26,1} },
			{ ["X"] = 76.90, ["Y"] = 41.30, ["Size"] = 30, ["L"] = {36,26,1} },
			{ ["X"] = 69.70, ["Y"] = 37.20, ["Size"] = 40, ["L"] = {1,36,26} },
			{ ["X"] = 24.10, ["Y"] = 34.70, ["Size"] = 40, ["L"] = {1,36,26} },
	},
	zDesolace = { L["Desolace"],
			{ ["X"] = 63.00, ["Y"] = 45.80, ["Size"] = 50, ["L"] = {29,8,22} },
			{ ["X"] = 72.60, ["Y"] = 72.20, ["Size"] = 80, ["L"] = {22,29,8} },
			{ ["X"] = 41.50, ["Y"] = 18.50, ["Size"] = 70, ["O"] = {24,26,19,11} },
			{ ["X"] = 23.50, ["Y"] = 77.00, ["Size"] = 90, ["O"] = {24,19,11,26} },
	},
	zDragonblight = { L["Dragonblight"],
			{ ["X"] = 11.50, ["Y"] = 46.00, ["Size"] = 35, ["L"] = {6,11,31}, ["O"] = {20} },
			{ ["X"] = 40.10, ["Y"] = 67.90, ["Size"] = 50, ["L"] = {6,11,31}, ["O"] = {20} },
			{ ["X"] = 63.80, ["Y"] = 73.90, ["Size"] = 30, ["L"] = {6,11,31}, ["O"] = {20} },
			{ ["X"] = 88.50, ["Y"] = 55.70, ["Size"] = 30, ["L"] = {6,11,31}, ["O"] = {20} },
			{ ["X"] = 34.00, ["Y"] = 81.90, ["Size"] = 40, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 49.60, ["Y"] = 76.40, ["Size"] = 70, ["O"] = {23,14,4,7,20} },
	},
	zDunMorogh = { L["Dun Morogh"],
			{ ["X"] = 76.70, ["Y"] = 40.50, ["Size"] = 20, ["L"] = {28,7,8} },
			{ ["X"] = 83.70, ["Y"] = 52.00, ["Size"] = 30, ["L"] = {28,8,7} },
			{ ["X"] = 17.70, ["Y"] = 74.60, ["Size"] = 20, ["L"] = {28,8,7} },
	},
	zDurotar = { L["Durotar"],
			{ ["X"] = 36.80, ["Y"] = 64.50, ["Size"] = 60, ["L"] = {7,28,34}, },
			{ ["X"] = 56.80, ["Y"] = 10.30, ["Size"] = 50, ["L"] = {7,28} },
			{ ["X"] = 38.70, ["Y"] = 35.00, ["Size"] = 50, ["L"] = {7,28} },
			{ ["X"] = 57.50, ["Y"] = 78.10, ["Size"] = 50, ["O"] = {25} },
	},
	zDuskwood = { L["Duskwood"],
			{ ["X"] = 57.10, ["Y"] = 13.00, ["Size"] = 30, ["L"] = {8,28} },
			{ ["X"] = 11.40, ["Y"] = 28.00, ["Size"] = 50, ["L"] = {8,28,34} },
	},
	zDustwallowMarsh = { L["Dustwallow Marsh"],
			{ ["X"] = 35.75, ["Y"] = 59.20, ["Size"] = 50, ["L"] = {29,8,22} },
			{ ["X"] = 51.54, ["Y"] = 21.00, ["Size"] = 40, ["L"] = {29,8,22} },
			{ ["X"] = 60.7, ["Y"] = 62.30, ["Size"] = 100, ["L"] = {29,8}, ["O"] = {24,19,26,11} },
	},
	zEasternPlaguelands = { L["Eastern Plaguelands"],
			{ ["X"] = 1.60, ["Y"] = 31.10, ["Size"] = 20, ["L"] = {33,29}, ["O"] = {22,19}, ["Notes"] = {12} },
			{ ["X"] = 10.10, ["Y"] = 36.10, ["Size"] = 20, ["L"] = {33,29}, ["O"] = {22,19}, ["Notes"] = {12} },
			{ ["X"] = 24.60, ["Y"] = 16.10, ["Size"] = 40, ["L"] = {33,29,22}, ["O"] = {22,19}, ["Notes"] = {12} },
			{ ["X"] = 56.50, ["Y"] = 73.60, ["Size"] = 50, ["L"] = {33,29,22}, ["O"] = {22,19}, ["Notes"] = {12} },
	},
	zEchoIsles = { L["Echo Isles"],			-- No parenthesis
			{ ["X"] = 42.40, ["Y"] = 52.60, ["Size"] = 20, ["O"] = {25} },
			{ ["X"] = 55.70, ["Y"] = 22.20, ["Size"] = 20, ["O"] = {25} },
			{ ["X"] = 35.50, ["Y"] = 50.00, ["Size"] = 100, ["L"] = {7,28} },
			{ ["X"] = 67.20, ["Y"] = 55.20, ["Size"] = 20, ["L"] = {7,28} },
	},
	zElwynnForest = { L["Elwynn Forest"],
			{ ["X"] = 53.00, ["Y"] = 8.30, ["Size"] = 20, ["L"] = {7,28} },
			{ ["X"] = 54.30, ["Y"] = 16.80, ["Size"] = 20, ["L"] = {7,28} },
			{ ["X"] = 54.00, ["Y"] = 36.70, ["Size"] = 30, ["L"] = {7,28} },
			{ ["X"] = 31.80, ["Y"] = 63.20, ["Size"] = 40, ["L"] = {7,28} },
			{ ["X"] = 48.60, ["Y"] = 65.30, ["Size"] = 50, ["L"] = {7,28} },
			{ ["X"] = 74.40, ["Y"] = 59.10, ["Size"] = 35, ["L"] = {7,28} },
			{ ["X"] = 90.50, ["Y"] = 36.20, ["Size"] = 60, ["L"] = {7,28} },
	},
	zEversongWoods = { L["Eversong Woods"],
			{ ["X"] = 64.20, ["Y"] = 84.80, ["Size"] = 50, ["L"] = {7,28} },
			{ ["X"] = 63.40, ["Y"] = 71.20, ["Size"] = 30, ["O"] = {25} },
			{ ["X"] = 58.00, ["Y"] = 66.50, ["Size"] = 30, ["O"] = {25} },
			{ ["X"] = 39.60, ["Y"] = 59.30, ["Size"] = 30, ["O"] = {25} },
	},
	zFelwood = { L["Felwood"],
			{ ["X"] = 42.40, ["Y"] = 47.70, ["Size"] = 30, ["L"] = {33,29}, ["O"] = {22,29,19}, ["Notes"] = {12} },
	},
	zFeralas = { L["Feralas"],
			{ ["X"] = 51.26, ["Y"] = 10.51, ["Size"] = 90, ["L"] = {22,33}, ["O"] = {30,22,29}, ["Notes"] = {12} },
			{ ["X"] = 44.77, ["Y"] = 12.06, ["Size"] = 30, ["L"] = {29,8} },
			{ ["X"] = 45.48, ["Y"] = 55.68, ["Size"] = 70, ["O"] = {24,26,11,19} },
	},
	zGhostlands = { L["Ghostlands"],
			{ ["X"] = 30.80, ["Y"] = 14.00, ["Size"] = 80, ["L"] = {28,8,7} },
			{ ["X"] = 71.30, ["Y"] = 64.70, ["Size"] = 40, ["L"] = {28,8,7} },
			{ ["X"] = 71.80, ["Y"] = 19.00, ["Size"] = 80, ["L"] = {28,8,7} },
			{ ["X"] = 62.90, ["Y"] = 55.60, ["Size"] = 40, ["O"] = {21,25,19} },
			{ ["X"] = 67.60, ["Y"] = 60.90, ["Size"] = 20, ["O"] = {21,25,19} },
			{ ["X"] = 66.70, ["Y"] = 64.60, ["Size"] = 20, ["O"] = {21,25,19} },
			{ ["X"] = 14.00, ["Y"] = 62.50, ["Size"] = 50, ["O"] = {21,25,19} },
	},
	zGrizzlyHills = { L["Grizzly Hills"],
			{ ["X"] = 86.20, ["Y"] = 30.50, ["Size"] = 10, ["L"] = {6,12,18}, ["O"] = {20} },
			{ ["X"] = 61.90, ["Y"] = 51.10, ["Size"] = 40, ["L"] = {18,6,12}, ["O"] = {20} },
			{ ["X"] = 39.90, ["Y"] = 33.30, ["Size"] = 40, ["L"] = {18,6,12}, ["O"] = {20} },
			{ ["X"] = 24.30, ["Y"] = 75.50, ["Size"] = 30, ["L"] = {18,6,12}, ["O"] = {20} },
			{ ["X"] = 84.00, ["Y"] = 28.50, ["Size"] = 40, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 77.80, ["Y"] = 37.60, ["Size"] = 30, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 14.90, ["Y"] = 86.60, ["Size"] = 60, ["O"] = {23,14,4,7,20} },
	},
	zHellfirePeninsula = { L["Hellfire Peninsula"],
			{ ["X"] = 40.60, ["Y"] = 32.10, ["Size"] = 30, ["L"] = {13,2} },
			{ ["X"] = 39.20, ["Y"] = 41.60, ["Size"] = 30, ["L"] = {13,2} },
	},
	zHillsbradFoothills = { L["Hillsbrad Foothills"],
			{ ["X"] = 56.80, ["Y"] = 61.50, ["Size"] = 30, ["L"] = {8,28} },
			{ ["X"] = 49.50, ["Y"] = 75.80, ["Size"] = 30, ["O"] = {21,19,11} },
			{ ["X"] = 30.20, ["Y"] = 84.50, ["Size"] = 30, ["O"] = {21,19,11} },
	},
	zHowlingFjord = { L["Howling Fjord"],
			{ ["X"] = 27.00, ["Y"] = 38.80, ["Size"] = 40, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 16.90, ["Y"] = 46.40, ["Size"] = 40, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 38.10, ["Y"] = 74.60, ["Size"] = 20, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 55.00, ["Y"] = 82.00, ["Size"] = 30, ["O"] = {23,14,4,7,20} },
			{ ["X"] = 61.90, ["Y"] = 53.40, ["Size"] = 60, ["L"] = {6,18,12}, ["O"] = {20} },
			{ ["X"] = 63.60, ["Y"] = 21.90, ["Size"] = 120, ["L"] = {12,6,18}, ["O"] = {20} },
	},
	zHrothgarsLanding = { L["Hrothgar's Landing"],
			{ ["X"] = 41.90, ["Y"] = 10.60, ["Size"] = 30, ["O"] = {23,4,14,7} },
			{ ["X"] = 53.050, ["Y"] = 31.40, ["Size"] = 60, ["O"] = {23,4,14,7} },
	},
	zIcecrown = { L["Icecrown"],
			{ ["X"] = 74.40, ["Y"] = 10.10, ["Size"] = 30, ["O"] = {23,4,14} },
			{ ["X"] = 79.30, ["Y"] = 15.10, ["Size"] = 20, ["O"] = {23,4,14} },
			{ ["X"] = 22.30, ["Y"] = 57.30, ["Size"] = 20, ["O"] = {23,4,14,7} },
			{ ["X"] = 35.90, ["Y"] = 16.40, ["Size"] = 20, ["O"] = {23,4,14,7} },
			{ ["X"] = 53.90, ["Y"] = 11.10, ["Size"] = 20, ["O"] = {23,4,14,7} },
			{ ["X"] = 6.50, ["Y"] = 46.90, ["Size"] = 40, ["O"] = {23,4,14,7} },
			{ ["X"] = 9.00, ["Y"] = 59.00, ["Size"] = 40, ["O"] = {23,4,14,7} },
	},
	zIronforge = { L["Ironforge"],			-- No parenthesis
			{ ["X"] = 47.00, ["Y"] = 19.20, ["Size"] = 90, ["L"] = {28,7,8} },
	},
	zIsleOfQuelDanas = { L["Isle of Quel'Danas"],	-- No parenthesis
			{ ["X"] = 60.80, ["Y"] = 59.80, ["Size"] = 30, ["O"] = {12,15} },
			{ ["X"] = 63.90, ["Y"] = 52.20, ["Size"] = 20, ["O"] = {12,15} },
			{ ["X"] = 58.10, ["Y"] = 87.80, ["Size"] = 50, ["O"] = {12,15} },
			{ ["X"] = 45.00, ["Y"] = 75.10, ["Size"] = 50, ["O"] = {12,15} },
			{ ["X"] = 40.20, ["Y"] = 57.00, ["Size"] = 30, ["O"] = {12,15} },
			{ ["X"] = 47.60, ["Y"] = 23.50, ["Size"] = 30, ["O"] = {12,15} },
			{ ["X"] = 55.60, ["Y"] = 34.00, ["Size"] = 30, ["O"] = {12,15} },
	},
	zKelpTharForest = { L["Kelp'thar Forest"],
			{ ["X"] = 56.30, ["Y"] = 30.70, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {7} },
			{ ["X"] = 63.20, ["Y"] = 60.90, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {7} },
			{ ["X"] = 35.20, ["Y"] = 19.70, ["Size"] = 20, ["O"] = {18,2}, ["Notes"] = {5} },
			{ ["X"] = 42.70, ["Y"] = 28.50, ["Size"] = 20, ["O"] = {18,2}, ["Notes"] = {5} },
			{ ["X"] = 44.90, ["Y"] = 23.50, ["Size"] = 20, ["O"] = {18,2}, ["Notes"] = {6} },
	},
	zLochModan = { L["Loch Modan"],
			{ ["X"] = 48.50, ["Y"] = 52.00, ["Size"] = 75, ["L"] = {28,27,34,8,7} },
	},
	zMoonglade = { L["Moonglade"],
			{ ["X"] = 60.00, ["Y"] = 58.80, ["Size"] = 90, ["L"] = {22,33}, ["O"] = {22,29,19}, ["Notes"] = {12,14} },
			{ ["X"] = 56.90, ["Y"] = 42.80, ["Size"] = 60, ["L"] = {22,33}, ["O"] = {22,29,19}, ["Notes"] = {12,14} },
	},
	zMountHyjal = { L["Mount Hyjal"],
			{ ["X"] = 71.50, ["Y"] = 74.25, ["Size"] = 80, ["L"] = {30,36,41}, ["Notes"] = {10} },
			{ ["X"] = 25.20, ["Y"] = 56.70, ["Size"] = 50, ["L"] = {36,30,41} },
			{ ["X"] = 39.70, ["Y"] = 30.30, ["Size"] = 50, ["L"] = {36,30,41} },
			{ ["X"] = 61.10, ["Y"] = 26.60, ["Size"] = 55, ["L"] = {41,30,36} },
	},
	zMulgore = { L["Mulgore"],
			{ ["X"] = 52.05, ["Y"] = 52.50, ["Size"] = 70, ["L"] = {7,28}, },
	},
	zNagrand = { L["Nagrand"],
			{ ["X"] = 75.60, ["Y"] = 54.20, ["Size"] = 40, ["L"] = {2,14,25} },
			{ ["X"] = 76.00, ["Y"] = 78.00, ["Size"] = 60, ["L"] = {2,14,25} },
			{ ["X"] = 34.10, ["Y"] = 49.80, ["Size"] = 115, ["L"] = {2,14,25} },
	},
	zNetherstorm = { L["Netherstorm"],
			{ ["X"] = 46.40, ["Y"] = 53.00, ["Size"] = 35, ["L"] = {2,14,25} },
	},
	zNewTinkertown = { L["New Tinkertown"],		-- No parenthesis
			{ ["X"] = 63.40, ["Y"] = 38.30, ["Size"] = 30, ["L"] = {7,28}, },
	},
	zNorthernBarrens = { L["Northern Barrens"],
			{ ["X"] = 71.60, ["Y"] = 60.40, ["Size"] = 60, ["L"] = {7,28,34}, },
			{ ["X"] = 37.30, ["Y"] = 45.70, ["Size"] = 40, ["L"] = {28,10,8,7}, },
			{ ["X"] = 40.50, ["Y"] = 73.70, ["Size"] = 55, ["L"] = {10,28,8}, },
			{ ["X"] = 56.00, ["Y"] = 80.00, ["Size"] = 40, ["L"] = {10,28,8,7}, },
			{ ["X"] = 71.60, ["Y"] = 82.00, ["Size"] = 60, ["O"] = {21,25,19,11} },
	},
	zNorthshire = { L["Northshire"],			-- No parenthesis
			{ ["X"] = 54.50, ["Y"] = 33.80, ["Size"] = 80, ["L"] = {7,28} },
	},
	zNorthernStranglethorn = { L["Northern Stranglethorn"],
			{ ["X"] = 25.80, ["Y"] = 18.50, ["Size"] = 20, ["L"] = {8,28} },
			{ ["X"] = 37.70, ["Y"] = 17.00, ["Size"] = 30, ["L"] = {28,8,34} },
			{ ["X"] = 81.50, ["Y"] = 28.40, ["Size"] = 90, ["L"] = {8,28} },
			{ ["X"] = 86.50, ["Y"] = 62.60, ["Size"] = 20, ["L"] = {8,28} },
			{ ["X"] = 78.00, ["Y"] = 72.50, ["Size"] = 20, ["L"] = {8,28} },
			{ ["X"] = 41.30, ["Y"] = 59.10, ["Size"] = 80, ["O"] = {19,21,11} },
			{ ["X"] = 22.60, ["Y"] = 37.00, ["Size"] = 70, ["O"] = {21,19,11} },
	},
	zOrgrimmar = { L["Orgrimmar"],			-- No parenthesis
			{ ["X"] = 37.70, ["Y"] = 81.90, ["Size"] = 50, ["L"] = {28,7} },
			{ ["X"] = 45.70, ["Y"] = 45.90, ["Size"] = 50, ["L"] = {28,8,7} },
			{ ["X"] = 64.50, ["Y"] = 41.50, ["Size"] = 70, ["L"] = {28,8,7} },
	},
	zRedridgeMountains = { L["Redridge Mountains"],
			{ ["X"] = 18.70, ["Y"] = 12.70, ["Size"] = 35, ["L"] = {28,8,7} },
			{ ["X"] = 27.30, ["Y"] = 48.00, ["Size"] = 20, ["L"] = {8,28} },
			{ ["X"] = 52.30, ["Y"] = 53.40, ["Size"] = 20, ["L"] = {28,34,8} },
	},
	zRuinsOfGilneas = { L["Ruins of Gilneas"],	-- No parenthesis
			{ ["X"] = 47.80, ["Y"] = 37.80, ["Size"] = 60, ["O"] = {21,25,19} },
			{ ["X"] = 63.85, ["Y"] = 96.00, ["Size"] = 30, ["O"] = {21,25,19} },
			{ ["X"] = 61.30, ["Y"] = 52.00, ["Size"] = 60, ["O"] = {21,25,19} },
	},
	zRuinsOfGilneasCity = { L["Ruins of Gilneas City"],		-- No parenthesis. A sub-zone of Ruins of Gilneas, but has its own zone ID
			{ ["X"] = 47.10, ["Y"] = 49.70, ["Size"] = 230, ["O"] = {21,25,19} },
	},
	zShadowglen = { L["Shadowglen"],			-- No parenthesis
			{ ["X"] = 43.30, ["Y"] = 49.60, ["Size"] = 80, ["L"] = {8,28} },
	},
	zShadowmoonValley = { L["Shadowmoon Valley"],
			{ ["X"] = 56.90, ["Y"] = 68.30, ["Size"] = 10, ["L"] = {13} },
			{ ["X"] = 26.30, ["Y"] = 40.80, ["Size"] = 40, ["L"] = {13} },
			{ ["X"] = 16.90, ["Y"] = 29.50, ["Size"] = 30, ["L"] = {13} },
	},
	zShimmeringExpanse = { L["Shimmering Expanse"],
			{ ["X"] = 55.50, ["Y"] = 13.60, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {7} },
			{ ["X"] = 49.30, ["Y"] = 41.60, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {7} },
			{ ["X"] = 49.80, ["Y"] = 66.20, ["Size"] = 30, ["O"] = {18,10,2,9}, ["Notes"] = {5} },
			{ ["X"] = 57.10, ["Y"] = 75.80, ["Size"] = 30, ["O"] = {18,10,2,9}, ["Notes"] = {5} },
			{ ["X"] = 60.70, ["Y"] = 28.50, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {5} },
			{ ["X"] = 57.20, ["Y"] = 18.00, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {5} },
			{ ["X"] = 45.60, ["Y"] = 18.00, ["Size"] = 30, ["O"] = {18,2}, ["Notes"] = {5} },
	},
	zSholazarBasin = { L["Sholazar Basin"],
			{ ["X"] = 38.30, ["Y"] = 38.50, ["Size"] = 50, ["L"] = {3,32}, ["O"] = {20} },
			{ ["X"] = 33.00, ["Y"] = 27.60, ["Size"] = 60, ["L"] = {3,32}, ["O"] = {20} },
			{ ["X"] = 18.00, ["Y"] = 60.10, ["Size"] = 60, ["L"] = {3,32}, ["O"] = {20} },
			{ ["X"] = 39.20, ["Y"] = 63.20, ["Size"] = 40, ["L"] = {3,32}, ["O"] = {20} },
			{ ["X"] = 48.30, ["Y"] = 64.50, ["Size"] = 50, ["L"] = {3,32}, ["O"] = {20} },
			{ ["X"] = 46.10, ["Y"] = 81.70, ["Size"] = 20, ["L"] = {3,32}, ["O"] = {20} },
			{ ["X"] = 63.40, ["Y"] = 50.80, ["Size"] = 50, ["L"] = {3,32}, ["O"] = {20} },
	},
	zSilverpineForest = { L["Silverpine Forest"],
			{ ["X"] = 61.90, ["Y"] = 34.40, ["Size"] = 70, ["L"] = {28,8,7,34} },
			{ ["X"] = 40.50, ["Y"] = 72.00, ["Size"] = 40, ["O"] = {21,25,19} },
	},
	zSouthernBarrens = { L["Southern Barrens"],
			{ ["X"] = 49.35, ["Y"] = 39.70, ["Size"] = 20, ["L"] = {29,8}, },
			{ ["X"] = 68.75, ["Y"] = 40.75, ["Size"] = 40, ["O"] = {24,19,26,11} },
	},
	zStonetalonMountains = { L["Stonetalon Mountains"],
			{ ["X"] = 51.10, ["Y"] = 45.60, ["Size"] = 50, ["L"] = {8,22,28}, },
	},
	zStranglethornVale = { L["Stranglethorn Vale"],	-- Keep this
	},
	zStormwindCity = { L["Stormwind City"],		-- No parenthesis
			{ ["X"] = 37.50, ["Y"] = 64.80, ["Size"] = 60, ["L"] = {28,7,8} },
			{ ["X"] = 54.70, ["Y"] = 68.90, ["Size"] = 20, ["L"] = {28,7,8} },
			{ ["X"] = 74.80, ["Y"] = 18.30, ["Size"] = 70, ["L"] = {28,7,8} },
	},
	zSunstriderIsle = { L["Sunstrider Isle"],
			{ ["X"] = 52.70, ["Y"] = 68.00, ["Size"] = 50, ["O"] = {25} },
			{ ["X"] = 62.70, ["Y"] = 78.30, ["Size"] = 50, ["O"] = {25} },
	},
	zSwampOfSorrows = { L["Swamp of Sorrows"],
			{ ["X"] = 74.90, ["Y"] = 8.50, ["Size"] = 45, ["O"] = {5,27,16,3,28,26,11}, ["Notes"] = {13} },
			{ ["X"] = 86.30, ["Y"] = 57.30, ["Size"] = 30, ["O"] = {5,28,3,27,16}, ["Notes"] = {13} },
			{ ["X"] = 93.10, ["Y"] = 81.80, ["Size"] = 35, ["O"] = {5,28,3,16,27}, ["Notes"] = {13} },
			{ ["X"] = 62.60, ["Y"] = 56.50, ["Size"] = 30, ["O"] = {5,28,3,27,16,11}, ["Notes"] = {13} },
	},
	zTanaris = { L["Tanaris"],
			{ ["X"] = 58.24, ["Y"] = 37.47, ["Size"] = 140, ["O"] = {27,26,11,19,13,24,3,28,31}, ["Notes"] = {13,15} },
	},
	zTeldrassil = { L["Teldrassil"],
			{ ["X"] = 44.5, ["Y"] = 35.90, ["Size"] = 30, ["L"] = {7,28} },
			{ ["X"] = 51.70, ["Y"] = 62.50, ["Size"] = 30, ["L"] = {7,28} },
			{ ["X"] = 57.90, ["Y"] = 59.20, ["Size"] = 30, ["L"] = {7,28} },
			{ ["X"] = 55.50, ["Y"] = 88.60, ["Size"] = 50, ["O"] = {25} },
			{ ["X"] = 30.60, ["Y"] = 51.40, ["Size"] = 30, ["L"] = {28,8,7} },
	},
	zTerokkarForest = { L["Terokkar Forest"],
			{ ["X"] = 65.60, ["Y"] = 68.00, ["Size"] = 30, ["L"] = {2,20,40}, ["Notes"] = {1} },
			{ ["X"] = 55.70, ["Y"] = 51.40, ["Size"] = 20, ["L"] = {2,20,40}, ["Notes"] = {1} },
			{ ["X"] = 51.50, ["Y"] = 44.50, ["Size"] = 20, ["L"] = {2,20,40}, ["Notes"] = {1} },
			{ ["X"] = 66.40, ["Y"] = 78.70, ["Size"] = 60, ["L"] = {2,20,40,16}, ["Notes"] = {2,4} },
			{ ["X"] = 59.50, ["Y"] = 61.30, ["Size"] = 50, ["L"] = {2,20,40,16}, ["Notes"] = {2,4} },
			{ ["X"] = 46.90, ["Y"] = 40.60, ["Size"] = 40, ["L"] = {2,20,40,16}, ["Notes"] = {2,4} },
			{ ["X"] = 39.90, ["Y"] = 11.40, ["Size"] = 50, ["L"] = {2,20,40,16}, ["Notes"] = {2,4} },
	},
	zTheCapeOfStranglethorn = { L["The Cape of Stranglethorn"],
			{ ["X"] = 40.50, ["Y"] = 70.50, ["Size"] = 60, ["O"] = {24,26,11,19} },
			{ ["X"] = 43.30, ["Y"] = 56.80, ["Size"] = 30, ["O"] = {24,26,11,19} },
			{ ["X"] = 34.30, ["Y"] = 27.60, ["Size"] = 50, ["O"] = {24,11,26,19} },
			{ ["X"] = 39.50, ["Y"] = 76.90, ["Size"] = 20, ["O"] = {24,26,19} },
			{ ["X"] = 43.20, ["Y"] = 86.40, ["Size"] = 65, ["O"] = {24,26,11,19} },
			{ ["X"] = 62.90, ["Y"] = 45.90, ["Size"] = 30, ["O"] = {24,19,11,26} },
	},
	zTheExodar = { L["The Exodar"],
			{ ["X"] = 85.10, ["Y"] = 88.40, ["Size"] = 60, ["O"] = {25} },
	},
	zTheHinterlands = { L["The Hinterlands"],
			{ ["X"] = 58.00, ["Y"] = 79.60, ["Size"] = 60, ["L"] = {29,8} },
			{ ["X"] = 49.30, ["Y"] = 35.40, ["Size"] = 60, ["L"] = {29,8,22} },
			{ ["X"] = 30.50, ["Y"] = 45.20, ["Size"] = 40, ["L"] = {29,8,22} },
			{ ["X"] = 77.80, ["Y"] = 72.60, ["Size"] = 30, ["O"] = {24,19,11,26} },
			{ ["X"] = 82.30, ["Y"] = 40.80, ["Size"] = 30, ["O"] = {24,19,11,26} },
	},
	zTheLostIsles = { L["The Lost Isles"],		-- No parenthesis
			{ ["X"] = 71.30, ["Y"] = 65.10, ["Size"] = 80, ["O"] = {25} },
			{ ["X"] = 40.80, ["Y"] = 64.10, ["Size"] = 80, ["O"] = {25} },
	},
	zTheMaelstrom = { L["The Maelstrom"],
			{ ["X"] = 15.00, ["Y"] = 52.70, ["Size"] = 170, ["Sub"] = L["The Lost Isles"] },
			{ ["X"] = 50.90, ["Y"] = 28.20, ["Size"] = 150, ["Sub"] = L["Deepholm"] },
	},
	zTheStormPeaks = { L["The Storm Peaks"],
			{ ["X"] = 28.50, ["Y"] = 28.40, ["Size"] = 30, ["O"] = {23,4,14} },
	},
	zThousandNeedles = { L["Thousand Needles"],
			{ ["X"] = 78.34, ["Y"] = 72.27, ["Size"] = 80, ["O"] = {26,13,27,24,11,3,28,31}, ["Notes"] = {13,15} },
	},
	zThunderBluff = { L["Thunder Bluff"],
			{ ["X"] = 40.20, ["Y"] = 58.60, ["Size"] = 40, ["L"] = {28,7} },
			{ ["X"] = 24.80, ["Y"] = 19.90, ["Size"] = 110, ["L"] = {28,8,7} },
	},
	zTirisfalGlades = { L["Tirisfal Glades"],
			{ ["X"] = 49.60, ["Y"] = 54.00, ["Size"] = 55, ["L"] = {7,28,38} },
			{ ["X"] = 68.50, ["Y"] = 43.20, ["Size"] = 80, ["L"] = {7,28,38} },
			{ ["X"] = 38.80, ["Y"] = 33.60, ["Size"] = 30, ["O"] = {25} },
	},
	zTolBaradPeninsula = { L["Tol Barad Peninsula"],
			{ ["X"] = 76.00, ["Y"] = 64.20, ["Size"] = 20, ["O"] = {18,9,2,10} },
			{ ["X"] = 65.90, ["Y"] = 65.30, ["Size"] = 20, ["O"] = {18,9,2,10} },
			{ ["X"] = 58.80, ["Y"] = 82.30, ["Size"] = 20, ["O"] = {18,9,2,10} },
			{ ["X"] = 49.50, ["Y"] = 64.20, ["Size"] = 30, ["O"] = {18,9,2,10} },
			{ ["X"] = 38.10, ["Y"] = 48.00, ["Size"] = 40, ["O"] = {18,9,2,10} },
			{ ["X"] = 53.40, ["Y"] = 34.00, ["Size"] = 60, ["O"] = {18,9,2,10} },
			{ ["X"] = 67.60, ["Y"] = 20.70, ["Size"] = 70, ["O"] = {18,9,2,10} },
	},
	zTwilightHighlands = { L["Twilight Highlands"],
			{ ["X"] = 28.00, ["Y"] = 64.00, ["Size"] = 55, ["L"] = {36,41,23} },
			{ ["X"] = 70.60, ["Y"] = 58.40, ["Size"] = 60, ["O"] = {18,9,2} },
	},
	zUldum = { L["Uldum"],
			{ ["X"] = 54.52, ["Y"] = 40.03, ["Size"] = 100, ["L"] = {36,4,26} },
			{ ["X"] = 84.54, ["Y"] = 58.14, ["Size"] = 70, ["O"] = {18,10,9} },
	},
	zUnGoroCrater = { L["Un'Goro Crater"],
			{ ["X"] = 34.60, ["Y"] = 54.20, ["Size"] = 100, ["L"] = {22,33}, ["O"] = {30,29,22}, ["Notes"] = {12,14} },
	},
	zUndercity = { L["Undercity"],			-- No parenthesis
			{ ["X"] = 81.10, ["Y"] = 31.80, ["Size"] = 30, ["L"] = {28,38,8,7} },
			{ ["X"] = 47.30, ["Y"] = 16.00, ["Size"] = 100, ["L"] = {28,8,7,38} },
			{ ["X"] = 47.30, ["Y"] = 16.00, ["Size"] = 100, ["L"] = {28,8,7,38} },
			{ ["X"] = 65.90, ["Y"] = 44.00, ["Size"] = 80, ["L"] = {28,8,7,38} },
	},
	zVashjir = { L["Vashj'ir"],	-- Keep this
	},
	zWesternPlaguelands = { L["Western Plaguelands"],
			{ ["X"] = 41.40, ["Y"] = 76.50, ["Size"] = 30, ["L"] = {29,8,22} },
			{ ["X"] = 33.00, ["Y"] = 70.00, ["Size"] = 60, ["L"] = {29,8,22} },
			{ ["X"] = 63.00, ["Y"] = 78.90, ["Size"] = 60, ["L"] = {29,8,22} },
			{ ["X"] = 64.40, ["Y"] = 17.50, ["Size"] = 20, ["L"] = {33,29}, ["O"] = {22,19}, ["Notes"] = {9,12} },
	},
	zWestfall = { L["Westfall"],
			{ ["X"] = 70.60, ["Y"] = 70.70, ["Size"] = 20, ["L"] = {28,7,8,34} },
			{ ["X"] = 46.30, ["Y"] = 59.00, ["Size"] = 25, ["L"] = {28,7,8} },
			{ ["X"] = 33.20, ["Y"] = 87.80, ["Size"] = 85, ["O"] = {21,25,19} },
			{ ["X"] = 41.80, ["Y"] = 10.10, ["Size"] = 60, ["O"] = {21,25,19} },
	},
	zWetlands = { L["Wetlands"],
			{ ["X"] = 54.00, ["Y"] = 37.00, ["Size"] = 45, ["L"] = {8,28} },
			{ ["X"] = 62.20, ["Y"] = 60.60, ["Size"] = 80, ["L"] = {8,34,28} },
			{ ["X"] = 6.00, ["Y"] = 62.00, ["Size"] = 40, ["O"] = {21,19,11} },
			{ ["X"] = 11.70, ["Y"] = 40.90, ["Size"] = 30, ["O"] = {21,19,11} },
	},
	zWintergrasp = { L["Wintergrasp"],
			{ ["X"] = 77.00, ["Y"] = 43.30, ["Size"] = 80, ["L"] = {18,32,31,17} },
			{ ["X"] = 33.00, ["Y"] = 19.40, ["Size"] = 60, ["L"] = {18,32,31,17} },
			{ ["X"] = 28.20, ["Y"] = 49.20, ["Size"] = 80, ["L"] = {18,32,31,17} },
			{ ["X"] = 58.30, ["Y"] = 74.10, ["Size"] = 70, ["L"] = {18,32,31,17} },
	},
	zWinterspring = { L["Winterspring"],
			{ ["X"] = 31.77, ["Y"] = 49.8, ["Size"] = 30, ["L"] = {33}, ["O"] = {30,29,22}, ["Notes"] = {12,14} },
	},
	zZangarmarsh = { L["Zangarmarsh"],
			{ ["X"] = 14.10, ["Y"] = 51.50, ["Size"] = 80, ["L"] = {2,42,40} },
			{ ["X"] = 58.10, ["Y"] = 41.20, ["Size"] = 80, ["L"] = {2,42,40,20}, ["Notes"] = {1} },
			{ ["X"] = 75.00, ["Y"] = 68.50, ["Size"] = 80, ["L"] = {2,42,40,20}, ["Notes"] = {1} },
	},
	zZulDrak = { L["Zul'Drak"],
			{ ["X"] = 38.10, ["Y"] = 84.30, ["Size"] = 65, ["L"] = {6,11,31}, ["O"] = {20} },
			{ ["X"] = 44.60, ["Y"] = 64.70, ["Size"] = 30, ["L"] = {6,11,31}, ["O"] = {20} },
			{ ["X"] = 54.20, ["Y"] = 47.60, ["Size"] = 50, ["L"] = {6,11,31}, ["O"] = {20} },
	},
}

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Zone & Fish Notes
--		=================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local zoneNotes = {
	"~L20: 18:00-24:00",
	"~L20: ".. L["Pools or"].. " 18:00-24:00 ".. L["open water"], 
	"~L20: ".. L["Brackish Mixed Schools"],
	"~L16: ".. L["Only pools"],
	L["Above!"],
	L["Inside a ship"],
	L["Inside a cave"],
	"",
	L["Stand in the Western Plaguelands, facing north. Game uses the Eastern Plaguelands fishing profile"],
	"~L30: ".. L["So many schools in this area"],				-- 10th
	"~L15: ".. L["The Eventide"],
	"~L33: ".. L["Not:"].. " 12:00-18:00. ".. L["Best:"].. " 00:00-06:00",
	"~O28: ".. L["Open water. 20th March to 23rd September."].. " ".. L["Best:"].. " 12:00-18:00. ".. 
		L["Not:"].. " 00:00-06:00",
	"~O29: ".. L["Best:"].. " 12:00 to 18:00. ".. L["Not:"].. " 00:00-06:00",
	"~O31: ".. L["Open water. 23rd September to 20th March."].. " ".. L["Best:"].. " 12:00-18:00. ".. L["Not:"].. " 00:00-06:00",
}

--[[--------------------------------------------------------------------------------------------------------------------------------------
--
--		Fish Indexes	( My Notes )
--		============
--
------------------------------------------------------------------------------------------------------------------------------------------

Limnologist:
===========

1 = Albino Cavefish		2 = Barbed Gill Trout		3 = Barrelhead Goby		4 = Blackbelly Mudfish
5 = Bloodfin Catfish		6 = Bonescale Snapper		7 = Brilliant Smallfish		8 = Bristle Whisker Catfish
9 = Crescent-Tail Skullfish	10 = Deviate Fish		11 = Dragonfin Angelfish	12 = Fangtooth Herring
13 = Felblood Snapper		14 = Figluster's Mudfish	15 Fountain Goldfish		16 Furious Crawdad
17 = Giant Darkwater Clam	18 = Glacial Salmon		19 = Glassfin Minnow		20 = Golden Darter
21 = Goldenscale Vendorfish	22 = Greater Sagefish		23 = Highland Guppy		24 = Huge Spotted Feltail
25 = Icefin Bluefish		26 = Lavascale Catfish		27 = Loch Frenzy		28 = Longjaw Mud Snapper
29 = Mithril Head Trout		30 = Mountain Trout		31 = Musselback Sculpin		32 = Nettlefish
33 = Nightfin Snapper		34 = Sagefish			35 = Sewer Carp			36 = Sharptooth
37 = Shimmering Minnow		38 = Sickly Looking Fish	39 = Slippery Eel		40 = Spotted Feltail
41 = Striped Lurker		42 = Zangarian Sporefish
}

Oceanographer:
=============

1 = Abyssal Clam		2 = Algaefin Rockfish		3 = Big-mouth Clam		4 = Borean Man O' War
5 = Darkclaw Lobster		6 = Darkshore Grouper		7 = Darkwater Clam		8 Deep Sea Monsterbelly
9 = Deepsea Sagefish		10 = Fathom Eel			11 = Firefin Snapper		12 = Giant Sunfish
13 = Glossy Mightfish		14 = Imperial Manta Ray		15 = Jaggal Clam		16 = Large Raw Mightfish
17 = Moonglow Cuttlefish	18 = Murglesnout		19 = Oily Blackmouth		20 = Pygmy Suckerfish
21 = Rainbow Fin Albacore	22 = Redgill			23 = Rockfin Grouper		24 = Rockscale Cod
25 = Slitherskin Mackerel	26 = Spotted Yellowtail		27 = Stonescale Eel		28 = Summer Bass
29 = Sunscale Salmon		30 = Whitescale Salmon		31 = Winter Squid

]]

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		World Map Neighbouring Zone Tooltips
--		====================================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local pc_colour_Needed	= "\124cFFFF4500"
local pc_colour_Done	= "\124cFF7FFF00"

local function FillFishTables( self, areaName )

	local z;
	local fishL, fishO = {}, {}

	for k,v in pairs( zones ) do
		if v[1] == areaName then
			z = k
			break
		end
	end
	if not z then
		return fishL, fishO
	end

	for i = 2, #zones[z] do
		if zones[z][i].L then
			for j = 1, #zones[z][i].L do
				for k = 1, #fishL do
					if fishL[k] == zones[z][i].L[j] then break end
					if k == #fishL then insert( fishL, zones[z][i].L[j] ) end
				end
				if #fishL == 0 then insert( fishL, zones[z][i].L[j] ) end
			end
		end
		if zones[z][i].O then
			for j = 1, #zones[z][i].O do
				for k = 1, #fishO do
					if fishO[k] == zones[z][i].O[j] then break end
					if k == #fishO then insert( fishO, zones[z][i].O[j] ) end
				end
				if #fishO == 0 then insert( fishO, zones[z][i].O[j] ) end
			end
		end
	end

	return fishL, fishO
end

local function ShowGameTooltip( self, fishL, fishO, areaName )

	local achieveN, achieveC = "", ""
	local titleL = false

	for i = 1, #fishL do
		local name, _, completed = GetAchievementCriteriaInfo( 5478, fishL[i] )
		if ( completed == true ) then
			achieveC = ( achieveC == "" ) and name or ( achieveC.. " / ".. name )
		else
			achieveN = ( achieveN == "" ) and name or ( achieveN.. " / ".. name )
		end
	end

	if ( achieveN ~= "" ) then
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:AddDoubleLine( pc_colour_Prefix.. addonName, pc_colour_PlainText.. " ".. areaName )
		GameTooltip:AddLine( pc_colour_Highlight.. L["The Limnologist"] )
		titleL = true
		GameTooltip:AddLine( pc_colour_Needed.. achieveN )
	end
	if ( achieveC ~= "" ) and ( CodDoBatterDB.showTooltipCompleted == "y" ) then
		if ( titleL == false ) then
			GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
			GameTooltip:AddDoubleLine( pc_colour_Prefix.. addonName, pc_colour_PlainText.. " ".. areaName )
			GameTooltip:AddLine( pc_colour_Highlight.. L["The Limnologist"] )
			titleL = true
		end
		GameTooltip:AddLine( pc_colour_Done.. achieveC )
	end

	achieveN, achieveC = "", ""
	local titleO = false

	for i = 1, #fishO do
		local name, _, completed = GetAchievementCriteriaInfo( 5479, fishO[i] )
		if ( completed == true ) then
			achieveC = ( achieveC == "" ) and name or ( achieveC.. " / ".. name )
		else
			achieveN = ( achieveN == "" ) and name or ( achieveN.. " / ".. name )
		end
	end

	if ( achieveN ~= "" ) then
		if ( titleL == false ) then
			GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
			GameTooltip:AddDoubleLine( pc_colour_Prefix.. addonName, pc_colour_PlainText.. " ".. areaName )
		end
		GameTooltip:AddLine( pc_colour_Highlight.. L["The Oceanographer"] )
		titleO = true
		GameTooltip:AddLine( pc_colour_Needed.. achieveN )
	end
	if ( achieveC ~= "" ) and ( CodDoBatterDB.showTooltipCompleted == "y" ) then
		if ( titleO == false ) then
			if ( titleL == false ) then
				GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
				GameTooltip:AddDoubleLine( pc_colour_Prefix.. addonName, pc_colour_PlainText.. " ".. areaName )
			end
			GameTooltip:AddLine( pc_colour_Highlight.. L["The Oceanographer"] )
			titleO = true
		end
		GameTooltip:AddLine( pc_colour_Done.. achieveC )
	end

	if ( titleL == true ) or ( titleO == true ) then
		GameTooltip:Show()
		GameTooltip.codDoBatter = areaName
	end
end

local function ShowTooltipForArea( self, area, areaName )

	local fishL, fishO = {}, {}
	if type( area ) == "table" then
		for i = 1, #area do
			local fishLsub, fishOsub = FillFishTables( self, area[i] )
			for j = 1, #fishLsub do
				for k = 1, #fishL do
					if fishL[k] == fishLsub[j] then break end
					if k == #fishL then insert( fishL, fishLsub[j] ) end
				end
				if #fishL == 0 then insert( fishL, fishLsub[j] ) end
			end
			for j = 1, #fishOsub do
				for k = 1, #fishO do
					if fishO[k] == fishOsub[j] then break end
					if k == #fishO then insert( fishO, fishOsub[j] ) end
				end
				if #fishO == 0 then insert( fishO, fishOsub[j] ) end
			end
		end
		ShowGameTooltip( self, fishL, fishO, areaName )
	else
		fishL, fishO = FillFishTables( self, area )
		ShowGameTooltip( self, fishL, fishO, areaName )
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Event Handling & OnUpdate Handler
--		=================================
--
--------------------------------------------------------------------------------------------------------------------------------------------

--local function round( num, places )		-- round to nearest integer, ties round away from zero
--	local mult = 10 ^ ( places or 0 )
--	return modf( num * mult + ( ( num < 0 and -1 ) or 1 ) * 0.5 ) / mult
--end

local function OnUpdateHandler( self, elapsed )

	if ( WorldMapFrame:IsVisible() == false ) or ( CodDoBatterDB.on == "n" ) then
		if GameTooltip.codDoBatter then
			GameTooltip.codDoBatter = nil
			GameTooltip:Hide()
		end
		return
	end

	local mapAreaID = GetCurrentMapAreaID()

--print("mapAreaID=".. mapAreaID)

	if ( CodDoBatterDB.showNeighbouringZone == "n" ) then
		if ( mapAreaID == 13 ) or ( mapAreaID == 14 ) or ( mapAreaID == 466 ) or ( mapAreaID == 485 ) then
			-- Kalimdor, Eastern Kingdoms, Outland, Northrend
		else
			if GameTooltip.codDoBatter then
				GameTooltip.codDoBatter = nil
				GameTooltip:Hide()
			end
			return
		end
	elseif ( mapAreaID == 1007 ) or ( mapAreaID == 1015 ) or ( mapAreaID == 1021 ) or ( mapAreaID == 1033 ) then
		if GameTooltip.codDoBatter then
			GameTooltip.codDoBatter = nil
			GameTooltip:Hide()
		end
		return
	end

	local areaLabel = WorldMapFrameAreaLabel:GetText() or ""
	local areaName = match( areaLabel, "(.+)|c" ) or areaLabel
	local severalAreas;

--print( "areaLabel=".. (areaLabel or "nil") )
--print( "areaName=".. (areaName or "nil") )

	if ( areaName == "" ) then
		-- Note: Some characters even at max level might not see opposition Cities on the respective zone maps
		-- because as yet "undiscovered". This AddOn could but does not assist in this respect

		if ( mapAreaID == 751 ) then	
			-- The General Maelstrom map which includes Deepholm, Kezan and The Lost Isles
			-- New players cannot fish in Kezan as there is no fishing trainer
			local xCursor, yCursor = GetCursorPosition()
 			local scale = WorldMapDetailFrame:GetEffectiveScale()
			xCursor, yCursor = ( xCursor / scale ), ( yCursor / scale )
			local width, height = WorldMapDetailFrame:GetWidth(), WorldMapDetailFrame:GetHeight()
			local left, top = WorldMapDetailFrame:GetLeft(), WorldMapDetailFrame:GetTop()
			xCursor, yCursor = ( ( xCursor - left ) / width ), ( ( top - yCursor ) / height )

			local xDist = math.abs( zones.zTheMaelstrom[2].X / 100 - xCursor ) * width
			local yDist = math.abs( zones.zTheMaelstrom[2].Y / 100 - yCursor ) * height
			local hypotenuse = xDist ^ 2 + yDist ^ 2
			if ( hypotenuse <= ( zones.zTheMaelstrom[2].Size / 2 ) ^ 2 ) then
				areaName = zones.zTheLostIsles[1]
			else
				xDist = math.abs( zones.zTheMaelstrom[3].X / 100 - xCursor ) * width
				yDist = math.abs( zones.zTheMaelstrom[3].Y / 100 - yCursor ) * height
				hypotenuse = xDist ^ 2 + yDist ^ 2
				if ( hypotenuse <= ( zones.zTheMaelstrom[3].Size / 2 ) ^ 2 ) then
					areaName = zones.zDeepholm[1]
				end
			end
		end

	elseif ( areaName == zones.zStranglethornVale[1] ) then
		severalAreas = { zones.zNorthernStranglethorn[1], zones.zTheCapeOfStranglethorn[1] }

	elseif ( areaName == zones.zVashjir[1] ) then
		severalAreas = { zones.zAbyssalDepths[1], zones.zShimmeringExpanse[1], zones.zKelpTharForest[1] }

	elseif ( areaName == zones.zNagrand ) or ( areaName == zones.zShadowmoonValley ) then
 		local mapContinentID = GetCurrentMapContinent()
		if ( mapContinentID == 7 ) then	-- Draenor
			areaName = nil
		end
	end

	if severalAreas then
		if GameTooltip.codDoBatter then
			if ( GameTooltip.codDoBatter ~= areaName ) then
				GameTooltip.codDoBatter = nil
				GameTooltip:Hide()
				ShowTooltipForArea( self, severalAreas, areaName )
			end
		else
			ShowTooltipForArea( self, severalAreas, areaName )
		end

	elseif areaName then
		if GameTooltip.codDoBatter then
			if ( GameTooltip.codDoBatter ~= areaName ) then
				GameTooltip.codDoBatter = nil
				GameTooltip:Hide()
				ShowTooltipForArea( self, areaName, areaName )
			end
		else
			ShowTooltipForArea( self, areaName, areaName )
		end

	elseif GameTooltip.codDoBatter then
		GameTooltip.codDoBatter = nil
		GameTooltip:Hide()
	end
end

local eventFrame = CreateFrame( "Frame" )
eventFrame:SetScript( "OnUpdate", OnUpdateHandler )

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		World Map Zone Rings
--		====================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local function ShowOnWorldMap()

 	local mapWidth = WorldMapDetailFrame:GetWidth()
 	local mapHeight = WorldMapDetailFrame:GetHeight()
	local stopIndex = 0

	local function AdjustCoordinates( x, y )
		return (x - 50) * mapWidth / 100, (mapHeight / 2) - (mapHeight * y / 100)
	end

	if ( CodDoBatterDB.showIcons == "n" ) then
		for i = 1, totalSubZones do
			Ring[ i ]:Hide()
		end
		return
	end

	mapAreaID, brightness = GetCurrentMapAreaID(), 0

--print( "GetCurrentMapDungeonLevel()=".. GetCurrentMapDungeonLevel() )

	if ( mapAreaID == 504 ) and WorldMapFrame:IsVisible() then

		-- Northrend Dalaran always has a non-zero dungeon level AND we need both levels to show separately

		if ( GetCurrentMapDungeonLevel() == 1 ) then
			for i = 2, 4 do
				Ring[ i-1 ]:SetPoint( "CENTER", WorldMapButton, "CENTER", AdjustCoordinates( zones[ mapAreaByID[ mapAreaID ] ][ i ].X, zones[ mapAreaByID[ mapAreaID ] ][ i ].Y ) )
				Ring[ i-1 ]:SetSize( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size, zones[ mapAreaByID[ mapAreaID ] ][ i ].Size )
				brightness = ( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size > 140 ) and 0.3 or 
					( ( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size > 20 ) and ( 1 - zones[ mapAreaByID[ mapAreaID ] ][ i ].Size / 20 * 0.1 ) or 1 )
				Ring[ i-1 ].iconTexture:SetVertexColor( 0.1, 0.3, 1, brightness )
				Ring[ i-1 ]:Show()
			end
			for i = 4, totalSubZones do
				Ring[ i ]:Hide()
			end
		elseif ( GetCurrentMapDungeonLevel() == 2 ) then	-- The Underbelly
			for i = 1, 3 do
				Ring[ i ]:Hide()
			end
			for i = 5, 6 do
				Ring[ i-1 ]:SetPoint( "CENTER", WorldMapButton, "CENTER", AdjustCoordinates( zones[ mapAreaByID[ mapAreaID ] ][ i ].X, zones[ mapAreaByID[ mapAreaID ] ][ i ].Y ) )
				Ring[ i-1 ]:SetSize( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size, zones[ mapAreaByID[ mapAreaID ] ][ i ].Size )
				brightness = ( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size > 140 ) and 0.3 or 
					( ( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size > 20 ) and ( 1 - zones[ mapAreaByID[ mapAreaID ] ][ i ].Size / 20 * 0.1 ) or 1 )
				Ring[ i-1 ].iconTexture:SetVertexColor( 0.1, 0.3, 1, brightness )
				Ring[ i-1 ]:Show()
			end
			for i = 6, totalSubZones do
				Ring[ i ]:Hide()
			end
		end
		return
	end

	if ( mapAreaID == 321 ) then				-- Orgrimmar always has a non-zero dungeon level
		if ( GetCurrentMapDungeonLevel() > 1 ) then
			for i = 1, totalSubZones do
				Ring[ i ]:Hide()
			end
			return
		end
	elseif ( GetCurrentMapDungeonLevel() ~= 0 ) then
		for i = 1, totalSubZones do
			Ring[ i ]:Hide()
		end
		return
	elseif not WorldMapFrame:IsVisible() then
		return
	end

	if mapAreaByID[ mapAreaID ] then
		for i = 2, #zones[ mapAreaByID[ mapAreaID ] ] do
			Ring[ i-1 ]:SetPoint( "CENTER", WorldMapButton, "CENTER", AdjustCoordinates( zones[ mapAreaByID[ mapAreaID ] ][ i ].X, zones[ mapAreaByID[ mapAreaID ] ][ i ].Y ) )
			Ring[ i-1 ]:SetSize( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size, zones[ mapAreaByID[ mapAreaID ] ][ i ].Size )
			brightness = ( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size > 140 ) and 0.3 or 
				( ( zones[ mapAreaByID[ mapAreaID ] ][ i ].Size > 20 ) and ( 1 - zones[ mapAreaByID[ mapAreaID ] ][ i ].Size / 20 * 0.1 ) or 1 )
			Ring[ i-1 ].iconTexture:SetVertexColor( 0.1, 0.3, 1, brightness )
			Ring[ i-1 ]:Show()
		end
		stopIndex = #zones[ mapAreaByID[ mapAreaID ] ]
	else
		for i = 1, totalSubZones do
			Ring[ i ]:Hide()
		end
		return
	end
	if ( (stopIndex-1) < totalSubZones) then
		for i = stopIndex, totalSubZones do
			Ring[ i ]:Hide()
		end
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		World Map Ring Tooltips
--		=======================
--
--------------------------------------------------------------------------------------------------------------------------------------------

function CDB_WorldMapTooltipOnEnter( self )

	WorldMapTooltip:SetOwner( self, "ANCHOR_LEFT" )

	mapAreaID = GetCurrentMapAreaID()

	local achieveLN, achieveLC, achieveON, achieveOC, zoneName, notes = "", "", "", "", "", {}

	if mapAreaByID[ mapAreaID ] then
		if zones[ mapAreaByID[ mapAreaID ] ][ self.index ].L then
			for i = 1, #zones[ mapAreaByID[ mapAreaID ] ][ self.index ].L do
				local name, _, completed = GetAchievementCriteriaInfo( 5478, zones[ mapAreaByID[ mapAreaID ] ][ self.index ].L[ i ] )
				if completed then
					achieveLC = ( achieveLC == "" ) and name or ( achieveLC.. " / ".. name )
				else
					achieveLN = ( achieveLN == "" ) and name or ( achieveLN.. " / ".. name )
				end
			end
		end
		if zones[ mapAreaByID[ mapAreaID ] ][ self.index ].O then
			for i = 1, #zones[ mapAreaByID[ mapAreaID ] ][ self.index ].O do
				local name, _, completed = GetAchievementCriteriaInfo( 5479, zones[ mapAreaByID[ mapAreaID ] ][ self.index ].O[ i ] )
				if completed then
					achieveOC = ( achieveOC == "" ) and name or ( achieveOC.. " / ".. name )
				else
					achieveON = ( achieveON == "" ) and name or ( achieveON.. " / ".. name )
				end
			end
		end
		zoneName = zones[ mapAreaByID[ mapAreaID ] ][1]
	end

	if ( achieveLN ~= "" ) or ( achieveLC ~= "" ) or ( achieveON ~= "" ) or ( achieveOC ~= "" ) then
		WorldMapTooltip:SetOwner(self, "ANCHOR_CURSOR")
		WorldMapTooltip:AddDoubleLine( pc_colour_Prefix.. addonName, pc_colour_PlainText.. " ".. zoneName )
	end
	if ( achieveLN ~= "" ) or ( achieveLC ~= "" ) then
		WorldMapTooltip:AddLine( pc_colour_Highlight.. L["The Limnologist"] )
		if ( achieveLN ~= "" ) then WorldMapTooltip:AddLine( pc_colour_Needed.. achieveLN ) end
		if ( achieveLC ~= "" ) then WorldMapTooltip:AddLine( pc_colour_Done.. achieveLC ) end
	end
	if ( achieveON ~= "" ) or ( achieveOC ~= "" ) then
		WorldMapTooltip:AddLine( pc_colour_Highlight.. L["The Oceanographer"] )
		if ( achieveON ~= "" ) then WorldMapTooltip:AddLine( pc_colour_Needed.. achieveON ) end
		if ( achieveOC ~= "" ) then WorldMapTooltip:AddLine( pc_colour_Done.. achieveOC ) end
	end

	if ( achieveLN ~= "" ) or ( achieveLC ~= "" ) or ( achieveON ~= "" ) or ( achieveOC ~= "" ) then
		if zones[ mapAreaByID[ mapAreaID ] ][ self.index ].Notes then
			for i = 1, #zones[ mapAreaByID[ mapAreaID ] ][ self.index ].Notes do
				local noteID = zones[ mapAreaByID[ mapAreaID ] ][ self.index ].Notes[i]
				local note = zoneNotes[ noteID ]
				if sub( note, 1, 1 ) == "~" then
					local achieve, fishIndex, replaced, fish = sub( note, 2, 2 ), tonumber( sub( note, 3, 4 ) ), sub( note, 1, 4 ), ""
					if ( achieve == "L" ) then
						fish = GetAchievementCriteriaInfo( 5478, fishIndex )
					else
						fish = GetAchievementCriteriaInfo( 5479, fishIndex )
					end
					note = stringGsub( note, replaced, fish )
				end
				WorldMapTooltip:AddLine( pc_colour_PlainText.. note )	
			end
		end
		WorldMapTooltip:Show()
	end
end

function CDB_WorldMapTooltipOnLeave( self )
	WorldMapTooltip:Hide()
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Event Handler
--		=============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local function CreateRing( i )

	local icon = CreateFrame( "Button", nil, WorldMapButton )			-- overlayFrame also works
	icon:HookScript( "OnEnter", CDB_WorldMapTooltipOnEnter )
	icon:HookScript( "OnLeave", CDB_WorldMapTooltipOnLeave )
	local iconTexture = icon:CreateTexture( nil, "OVERLAY" )
--	iconTexture:SetTexture( "Interface\\Common\\Portrait-Ring-WithBG-Highlight" )
	iconTexture:SetTexture( "Interface\\Minimap\\UI-QuestBlob-MinimapRing" )
--	iconTexture:SetTexture( "Interface\\Artifacts\\Artifacts-PerkRing-Final-Mask" )
	iconTexture:SetVertexColor( 0.1, 0.3, 1, 0.6 )	-- 0.6 is a bit dark for the other textures
	iconTexture:SetAllPoints()
	icon.iconTexture = iconTexture
	icon.index = i + 1
	icon:Hide()
	return icon
end

local firstTimeIn, recentTime = true, 0

local function EventHandler( self, event, ...)

	if ( event == "PLAYER_LOGIN" ) then
		if not CodDoBatterDB then CodDoBatterDB = {} end
		local onOff = CodDoBatterDB.on or "y"
		local showIcons = CodDoBatterDB.showIcons or "y"
		local showTooltipCompleted = CodDoBatterDB.showTooltipCompleted or "n"
		local showNeighbouringZone = CodDoBatterDB.showNeighbouringZone or "y"
		CodDoBatterDB = {}
		CodDoBatterDB.on = onOff
		CodDoBatterDB.showIcons = showIcons
		CodDoBatterDB.showTooltipCompleted = showTooltipCompleted
		CodDoBatterDB.showNeighbouringZone = showNeighbouringZone
		return
	end

	if ( firstTimeIn == true ) then

		if event ~= "PLAYER_ENTERING_WORLD" then return end

		for i = 1, totalSubZones do
			Ring[ i ] = CreateRing( i )
		end

		firstTimeIn = false
	end

	if ( event == "ZONE_CHANGED_NEW_AREA" ) then
		SetMapToCurrentZone()
		mapAreaID = GetCurrentMapAreaID()
		recentTime = 0
	end

	if ( recentTime + TOOLTIP_UPDATE_TIME ) < GetTime() then
		recentTime = GetTime()
		ShowOnWorldMap()
	end
end

eventFrame:RegisterEvent( "ZONE_CHANGED_NEW_AREA" )
eventFrame:RegisterEvent( "WORLD_MAP_UPDATE" )
eventFrame:RegisterEvent( "PLAYER_ENTERING_WORLD" )
eventFrame:RegisterEvent( "PLAYER_LOGIN" )
eventFrame:SetScript( "OnEvent", EventHandler )

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Summary of Outstanding
--		======================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local function ShowOutstanding( fishOrZone )

	local showL, showO = false, false
	local fish = fishOrZone and lower( fishOrZone ) or nil
	if ( fish == "l" ) then
		showL, fish = true, nil
	elseif ( fish == "o" ) then
		showO, fish = true, nil
	else
		showL, showO = true, true
	end

	local numCriteriaL = GetAchievementNumCriteria( 5478 )
	local titleL;

	if ( showL == true ) then
		for i = 1, numCriteriaL do
			local name, _, completed = GetAchievementCriteriaInfo( 5478, i )
			if ( fish and ( fish == lower( name ) ) ) or ( fish == nil ) then
				if ( completed == false ) then
					local achieveN = ""
					for k,v in pairs( zones ) do
						for j = 2, #v do
							if v[j].L then
								for m = 1, #v[j].L do
									if v[j].L[m] == i then
										if not match( achieveN, v[1] ) then
											achieveN = ( achieveN == "" ) and v[1] or ( achieveN.. " / ".. v[1] )
										end
									end
								end
							end
						end
					end
					if not titleL then
						printPC( pc_colour_Highlight.. "** ".. L["The Limnologist"].. " **" )
						titleL = true
					end
					printPC( pc_colour_Highlight.. name.. " ".. pc_colour_PlainText.. achieveN )
				end
			end
		end
	end

	local numCriteriaO = GetAchievementNumCriteria( 5479 )
	local titleO;

	if ( showO == true ) then
		for i = 1, numCriteriaO do
			local name, _, completed = GetAchievementCriteriaInfo( 5479, i )
			if ( fish and ( fish == lower( name ) ) ) or ( fish == nil ) then
				if ( completed == false ) then
					local achieveN = ""
					for k,v in pairs( zones ) do
						for j = 2, #v do
							if v[j].O then
								for m = 1, #v[j].O do
									if v[j].O[m] == i then
										if not match( achieveN, v[1] ) then
											achieveN = ( achieveN == "" ) and v[1] or ( achieveN.. " / ".. v[1] )
										end
									end
								end
							end
						end
					end
					if not titleO then
						printPC( pc_colour_Highlight.. "** ".. L["The Oceanographer"].. " **" )
					titleO = true
					end
					printPC( pc_colour_Highlight.. name.. " ".. pc_colour_PlainText.. achieveN )
				end
			end
		end
	end

	if ( titleL == true ) or ( titleO == true ) or ( fishOrZone == nil ) then return end

	-- must be a zone

	local achieveZ = ""
	local fishOrZoneLowered = lower( fishOrZone )
	for k,v in pairs( zones ) do
		if ( fishOrZoneLowered == lower( v[1] ) ) then
			for j = 2, #v do
				if v[j].L then
					for m = 1, #v[j].L do
						local name, _, completed = GetAchievementCriteriaInfo( 5478, v[j].L[m] )
						if ( completed == false ) then
							if not match( achieveZ, name ) then
								achieveZ = ( achieveZ == "" ) and name or ( achieveZ.. " / ".. name )
							end
						end
					end
				end
			end
			if ( achieveZ ~= "" ) then
				printPC( pc_colour_Highlight.. "** ".. L["The Limnologist"].. " **" )
				printPC( pc_colour_Highlight.. v[1].. " ".. pc_colour_PlainText.. achieveZ )
			end
			achieveZ = ""
			for j = 2, #v do
				if v[j].O then
					for m = 1, #v[j].O do
						local name, _, completed = GetAchievementCriteriaInfo( 5479, v[j].O[m] )
						if ( completed == false ) then
							if not match( achieveZ, name ) then
								achieveZ = ( achieveZ == "" ) and name or ( achieveZ.. " / ".. name )
							end
						end
					end
				end
			end
			if ( achieveZ ~= "" ) then
				printPC( pc_colour_Highlight.. "** ".. L["The Oceanographer"].. " **" )
				printPC( pc_colour_Highlight.. v[1].. " ".. pc_colour_PlainText.. achieveZ )
			end
		end
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Slash Commands
--		==============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local soundkitHandle = nil

SLASH_CodDoBatter1, SLASH_CodDoBatter2, SLASH_CodDoBatter3, SLASH_CodDoBatter4 = "/cdb", "/cod", "/batter", "/coddobatter"

SlashCmdList[ "CodDoBatter" ] = function( options )

	local thirdParm, secondParm, options1
	for v in gmatch( options, "(%S+)" ) do
		if not options1 then
			options1 = sub( lower(v),1,1 )
		elseif not secondParm then
			secondParm = v
		elseif not thirdParm then
			secondParm = secondParm.. " ".. v
			thirdParm = v
		else
			secondParm = secondParm.. " ".. v
			break
		end
	end

	if ( options1 == "?" ) or not options1 then
		printPC( pc_colour_Highlight .. L["Commands"].. ":\n"..
			pc_colour_Highlight.. "s l/o".. pc_colour_PlainText.. " = ".. L["Summary"].. " (l=".. L["The Limnologist"]..
				", o=".. L["The Oceanographer"].. ")\n"..
			pc_colour_Highlight.. "s {".. L["Fish"].. "}".. pc_colour_PlainText.. " = ".. L["Summary"].. "\n"..
			pc_colour_Highlight.. "s {".. L["Zone"].. "}".. pc_colour_PlainText.. " = ".. L["Summary"].. "\n"..
			pc_colour_Highlight.. "o".. pc_colour_PlainText.. " = ".. L["Turn on / off"].. ". (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "c".. pc_colour_PlainText.. " = ".. L["Show Completed"].. ". (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "n".. pc_colour_PlainText.. " = ".. L["Show Neighbouring Zone"].. ". (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "v".. pc_colour_PlainText.. " = ".. L["Show the version number"].. "\n")

	elseif ( options1 == "s" ) then
		ShowOutstanding( secondParm )

	elseif ( options1 == "o" ) then
		if secondParm then
			printPC( "/cdb o (".. ( ( CodDoBatterDB.on == "y" ) and L["On"] or L["Off"] ).. ")" )
		else
			CodDoBatterDB.on = ( CodDoBatterDB.on == "y" ) and "n" or "y"
			if ( CodDoBatterDB.on == "y" ) then
				printPC( L["On"] )
			else
				printPC( L["Off"] )
			end
		end

	elseif ( options1 == "c" ) then
		if secondParm then
			printPC( "/cdb c (".. ( ( CodDoBatterDB.showTooltipCompleted == "y" ) and L["Show"] or L["Hide"] ).. ")" )
		else
			CodDoBatterDB.showTooltipCompleted = ( CodDoBatterDB.showTooltipCompleted == "y" ) and "n" or "y"
			if ( CodDoBatterDB.showTooltipCompleted == "y" ) then
				printPC( "(".. L["Show"].. ")" )
			else
				printPC( "(".. L["Hide"].. ")" )
			end
		end

	elseif ( options1 == "n" ) then
		if secondParm then
			printPC( "/cdb n (".. ( ( CodDoBatterDB.showNeighbouringZone == "y" ) and L["Show"] or L["Hide"] ).. ")" )
		else
			CodDoBatterDB.showNeighbouringZone = ( CodDoBatterDB.showNeighbouringZone == "y" ) and "n" or "y"
			if ( CodDoBatterDB.showNeighbouringZone == "y" ) then
				printPC( "(".. L["Show"].. ")" )
			else
				printPC( "(".. L["Hide"].. ")" )
			end
		end

	elseif ( options1 == "v" ) then
		local version = GetAddOnMetadata( addonName, "Version" )
		printPC( L["Version: "].. pc_colour_Highlight.. version )

	-- Easter Eggs. Enjoy!

	elseif ( options1 == "z" ) then
		if ( soundkitHandle ) then
			StopSound( soundkitHandle, 5000 )
			soundkitHandle = nil
		else
			local soundSelection = random( 2 )
			if ( soundSelection == 1 ) then
				_, soundkitHandle = PlaySound( 13799 )		-- Storm Peaks Day
			else
				_, soundkitHandle = PlaySound( 22161 )		-- Vashj'ir
			end
		end
	end
end